"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var apollo_cache_inmemory_1 = require("apollo-cache-inmemory");
var redux_1 = require("redux");
var constants_1 = require("./constants");
var reducer_1 = require("./reducer");
var ReduxCache = (function (_super) {
    __extends(ReduxCache, _super);
    function ReduxCache(config, store) {
        if (config === void 0) { config = {}; }
        if (store === void 0) { store = undefined; }
        var _this = _super.call(this, config) || this;
        _this.store = store || redux_1.createStore(redux_1.combineReducers({ apollo: reducer_1.apolloReducer }));
        _this.reduxRootSelector = 'apollo';
        return _this;
    }
    ReduxCache.prototype.restore = function (data) {
        this.store.dispatch({
            type: constants_1.APOLLO_RESTORE,
            data: data
        });
        return this;
    };
    ReduxCache.prototype.extract = function (optimistic) {
        if (optimistic === void 0) { optimistic = false; }
        if (optimistic && this.optimistic.length > 0) {
            var patches = this.optimistic.map(function (opt) { return opt.data; });
            return Object.assign.apply(Object, [this.getReducer()].concat(patches));
        }
        return this.getReducer();
    };
    ReduxCache.prototype.read = function (query) {
        if (query.rootId && this.getReducer()[query.rootId] === undefined) {
            return null;
        }
        var options = {
            store: this.config.storeFactory(this.extract(query.optimistic)),
            query: this.transformDocument(query.query),
            variables: query.variables,
            rootId: query.rootId,
            fragmentMatcherFunction: this.config.fragmentMatcher.match,
            previousResult: query.previousResult,
            config: this.config,
        };
        return apollo_cache_inmemory_1.readQueryFromStore(options);
    };
    ReduxCache.prototype.write = function (write) {
        var data = this.config.storeFactory();
        apollo_cache_inmemory_1.writeResultToStore({
            dataId: write.dataId,
            result: write.result,
            variables: write.variables,
            document: this.transformDocument(write.query),
            store: data,
            dataIdFromObject: this.config.dataIdFromObject,
            fragmentMatcherFunction: this.config.fragmentMatcher.match,
        });
        this.store.dispatch({
            type: constants_1.APOLLO_WRITE,
            data: data.toObject()
        });
        this.broadcastWatches();
    };
    ReduxCache.prototype.evict = function (query) {
        throw new Error("evict() is not implemented on Redux Cache");
    };
    ReduxCache.prototype.reset = function () {
        this.store.dispatch({
            type: constants_1.APOLLO_RESET,
        });
        this.broadcastWatches();
        return Promise.resolve();
    };
    ReduxCache.prototype.getReducer = function () {
        return this.store.getState()[this.reduxRootSelector];
    };
    return ReduxCache;
}(apollo_cache_inmemory_1.InMemoryCache));
exports.ReduxCache = ReduxCache;
//# sourceMappingURL=reduxCache.js.map